@@ -1,6 +1,6 @@
 /*
 
- $Id: constants.h,v 1.42 2012/10/04 23:28:12 mp Exp $
+ $Id: constants.h,v 1.44 2014/06/12 01:44:07 mp Exp $
 
  AutoDock 
 
@@ -102,7 +102,7 @@ Copyright (C) 2009 The Scripps Research Institute. All rights reserved.
 #define MAX_RECORDS  128     /* Maximum number of PDBQ records =~1.5*MAX_ATOMS */
 #define MAX_NONBONDS 16384   /* 1048576   Maximum number of non-bonds in Small Molecule.*/
 #else
-#define MAX_RUNS     2000      /* Maximum number of runs. */
+#define MAX_RUNS     2000      /* Maximum number of runs. Used in Tests */
 #define MAX_ATOMS    2048     /* Maximum number of atoms in Small Molecule. */
 #define MAX_NBONDS    16     /* Maximum number of bonds per atom, in readPDBQT  */
 #define MAX_RECORDS  2048     /* Maximum number of PDBQ records =~1.5*MAX_ATOMS */
@@ -338,7 +338,7 @@ const struct {
  * Random numbers,                                                            * 
  *----------------------------------------------------------------------------*/
 
-// This is platform-independent RNG-based.
+// This is platform-independent RNG-based.  All are wrappers around ignlgi() - I think! MPique 2014
 #include "ranlib.h"
 #define local_random()      genunf(0., 1.)
 
@@ -352,7 +352,7 @@ const struct {
 #define random_sign        ( (local_random() < 0.5) ?  (-1.0) : (+1.0) )
 #define random_pm1()    ( 2.*local_random() - 1. ) /* ..."pm"="Plus or Minus" */
 #define Randpm1                random_pm1()
-#define RandpmPI        PI * Randpm1
+#define RandpmPI        (PI * Randpm1)
 #define random_pm(x)    ( (x) * Randpm1 )    /* ...random num. between +/- x; */
 /* Return a random number in the range a to b...  */
 #define random_range(a,b)    ( (a) + (((b)-(a))*local_random()) )
