@@ -1,6 +1,6 @@
 /*
 
- $Id: qmultiply.cc,v 1.19 2011/06/03 05:31:36 mp Exp $
+ $Id: qmultiply.cc,v 1.23 2014/06/20 23:03:52 mp Exp $
 
  AutoDock 
 
@@ -34,10 +34,8 @@ Copyright (C) 2009 The Scripps Research Institute. All rights reserved.
 #include <assert.h>
 #include "qmultiply.h"
 
-extern  FILE    *logFile;
-
-static Quat sidentityQuat = { 0., 0., 0., 1.}; // x,y,z,w
-static AxisAngle sidentityAxisAngle = { 1., 0., 0., 0.}; // x,y,z,angle
+static Quat sidentityQuat = { 0., 0., 0., 1.}; // x,y,z,w  const
+static AxisAngle sidentityAxisAngle = { 1., 0., 0., 0.}; // x,y,z,angle  const
 
 
 void qmultiply( /* not const */  Quat *const q, //result
@@ -277,7 +275,7 @@ raaToQuat( const Real raa[3], ConstReal angle )   // "Real"type signature
     return AxisAngleToQuat( input );
 }
 Quat
-raaToQuat( const double raa[3], ConstReal angle ) // "double" type signature
+raaDoubleToQuat( const double raa[3], const double angle ) // "double" type signature
 {
     // input axis need not be normalized
     AxisAngle input;
@@ -323,7 +321,7 @@ randomQuatByAmount( ConstReal amount )
     // amount is an angle in radians
     Quat q = randomQuat();
     AxisAngle aa = QuatToAxisAngle( q ); // will not be 3-element normalized
-    return axisRadianToQuat( q.x, q.y, q.z, amount );
+    return axisRadianToQuat( aa.nx, aa.ny, aa.nz, amount );
 }
 
 void print_q_reorient_message( FILE *const logFile, const Quat& q_reorient )
