@@ -1,6 +1,6 @@
 /*
 
- $Id: ranlib.cc,v 1.13 2012/10/15 17:48:28 mp Exp $
+ $Id: ranlib.cc,v 1.17 2014/06/12 01:44:08 mp Exp $
 
  AutoDock 
 
@@ -35,26 +35,32 @@ Copyright (C) 2009 The Scripps Research Institute. All rights reserved.
 #include "structs.h"
 #include "stop.h"
 
-/* this software can be configured to provide multiple concurrent
- random number generators (e.g., 32), however, AutoDock uses only one
- at present.  This number must also be defined in com.cc M Pique 2012-07
+/* this software is configured to provide multiple concurrent
+ random number generators (e.g., 32), AutoDock uses one per OpenMP thread
+ at present.  This number is NUMG and is defined in ranlib.h
+ The OpenMP code depends on all random numbers being derived from calls
+ the ignlgi_t(), the per-thread "Integer GeNerate LarGe Integer" in com.cc.
+ The function ignlgi() is a macro defined in com.h that calls ignlgi_t()
+ with the current thread number, or 0 if not running in parallel.
+ M Pique 2014-02
+ 
  */
-#define NUMG 1
 
 // C++ equivalents of Fortran functions:
 inline Real ABS(ConstReal x) { return ((x) >= 0 ? (x) : -(x)) ; }
 inline Real MIN(ConstReal a, ConstReal b) { return ((a) <= (b) ? (a) : (b)) ; }
 inline Real MAX(ConstReal a, ConstReal b) { return ((a) >= (b) ? (a) : (b)) ; }
-inline FourByteLong IABS(const FourByteLong x) { return ((x) >= 0 ? (x) : -(x)) ; } 
-inline FourByteLong IMIN(const FourByteLong a, const FourByteLong b) { return ((a) <= (b) ? (a) : (b)) ; }
-inline FourByteLong IMAX(const FourByteLong a, const FourByteLong b) { return ((a) >= (b) ? (a) : (b)) ; }
+inline long IABS(const long x) { return ((x) >= 0 ? (x) : -(x)) ; } 
+inline long IMIN(const long a, const long b) { return ((a) <= (b) ? (a) : (b)) ; }
+inline long IMAX(const long a, const long b) { return ((a) >= (b) ? (a) : (b)) ; }
 
+// functions used only internally within this source file:
 static Real sgamma(ConstReal a);
 static Real sexpo(void);
 static Real fsign(ConstReal num, ConstReal sign);
 static Real snorm(void);
-static FourByteLong ignbin(const FourByteLong n, ConstReal pp);
-static FourByteLong ignpoi(ConstReal mu);
+static FourByteLong ignbin(const int n, ConstReal pp);
+static FourByteLong ignpoi(ConstReal mu); // not used in AutoDock code
 
 
 static const Real expmax=89.0;
@@ -252,11 +258,7 @@ static Real gengam(ConstReal a, ConstReal r)
 **********************************************************************
 */
 {
-static Real gengam;
-
-    gengam = sgamma(r);
-    gengam /= a;
-    return gengam;
+    return sgamma(r)/a;
 }
 
 static Real genchi(ConstReal df)
@@ -276,7 +278,7 @@ static Real genchi(ConstReal df)
 **********************************************************************
 */
 {
-static Real genchi;
+Real genchi;
 
     if(!(df <= 0.0)) goto S10;
     fputs("DF <= 0 in GENCHI - ABORT",stderr);
@@ -309,10 +311,7 @@ Real genexp(ConstReal av) // not used in AutoDock code
 **********************************************************************
 */
 {
-static Real genexp;
-
-    genexp = sexpo()*av;
-    return genexp;
+    return sexpo()*av;
 }
 
 Real genf(ConstReal dfn, ConstReal  dfd) // not used in AutoDock code
@@ -410,7 +409,7 @@ static Real ae;
     }
 }
 
-void genmul(const FourByteLong n, const Real *const p, const FourByteLong ncat, /* not const */ FourByteLong *const ix) // not used in AutoDock code
+void genmul(const int n, const Real *const p, const int ncat, /* not const */ int *const ix) // not used in AutoDock code
 /*
 **********************************************************************
  
@@ -439,7 +438,7 @@ void genmul(const FourByteLong n, const Real *const p, const FourByteLong ncat,
 */
 {
 static Real prob,ptot,sum;
-static FourByteLong i,icat,ntot;
+static int i,icat,ntot;
     if(n < 0) stop("N < 0 in GENMUL");
     if(ncat <= 1) stop("NCAT <= 1 in GENMUL");
     ptot = 0.0F;
@@ -493,15 +492,12 @@ static Real gennch(ConstReal df, ConstReal xnonc)
 **********************************************************************
 */
 {
-static Real gennch;
-
     if(!(df <= 1.0 || xnonc < 0.0)) goto S10;
     fputs("DF <= 1 or XNONC < 0 in GENNCH - ABORT",stderr);
     fprintf(stderr,"Value of DF: %16.6E Value of XNONC%16.6E\n",df,xnonc);
     stop("ERROR in gennch");
 S10:
-    gennch = genchi(df-1.0)+pow(gennor(sqrt(xnonc),1.0),2.0);
-    return gennch;
+    return genchi(df-1.0)+pow(gennor(sqrt(xnonc),1.0),2.0);
 }
 
 Real gennf(ConstReal dfn, ConstReal dfd, ConstReal xnonc) // not used in AutoDock code
@@ -528,7 +524,7 @@ Real gennf(ConstReal dfn, ConstReal dfd, ConstReal xnonc) // not used in AutoDoc
 */
 {
 static Real gennf,xden,xnum;
-static FourByteLong qcond;
+static int qcond;
 
     qcond = dfn <= 1.0 || dfd <= 0.0 || xnonc < 0.0;
     if(!qcond) goto S10;
@@ -579,10 +575,7 @@ extern Real gennor(ConstReal av, ConstReal sd)
 **********************************************************************
 */
 {
-static Real gennor;
-
-    gennor = sd*snorm()+av;
-    return gennor;
+    return sd*snorm()+av;
 }
 
 void genprm(/* not const */ FourByteLong *const iarray, const int larray) // not used in AutoDock code
@@ -597,7 +590,7 @@ void genprm(/* not const */ FourByteLong *const iarray, const int larray) // not
 **********************************************************************
 */
 {
-static FourByteLong i,itmp,iwhich,D1,D2;
+FourByteLong i,itmp,iwhich,D1,D2;
 
     for (i=1,D1=1,D2=(larray-i+D1)/D1; D2>0; D2--,i+=D1) {
         iwhich = ignuin(i,larray);
@@ -620,79 +613,19 @@ extern Real genunf(ConstReal low, ConstReal high)
 **********************************************************************
 */
 {
-    static Real genunf;
     char msg[1000];
 
     if(!(low > high)) goto S10;
     sprintf(msg,"LOW > HIGH in GENUNF: LOW %16.6E HIGH: %16.6E\n",low,high);
     stop(msg);
 S10:
-    genunf = low+(high-low)*ranf();
-    return genunf;
+    return low+(high-low)*ranf();
 }
 
-extern void gscgn(const FourByteLong getset, FourByteLong *const g)
+static FourByteLong ignbin(const int n, ConstReal pp) // not used in AutoDock code
 /*
 **********************************************************************
-     void gscgn(FourByteLong getset,FourByteLong *g)
-                         Get/Set GeNerator
-     Gets or returns in G the number of the current generator
-                              Arguments
-     getset --> 0 Get
-                1 Set
-     g <-- Number of the current random number generator (1..NUMG)
-**********************************************************************
-*/
-{
-    static FourByteLong curntg = 1;
-    if(getset == 0) *g = curntg;
-    else if( *g > 0 && *g <= NUMG) curntg = *g;
-    else {
-	    char errmsg[200];
-            sprintf(errmsg, 
-	    "BUG: Generator number %ld out of range %d to %d in GSCGN",*g,1,NUMG);
-	    stop(errmsg);
-        }
-}
-
-extern void gsrgs(const FourByteLong getset, /* not const */ FourByteLong *const qvalue)
-/*
-**********************************************************************
-     void gsrgs(FourByteLong getset,FourByteLong *qvalue)
-               Get/Set Random Generators Set
-     Gets or sets whether random generators set (initialized).
-     Initially (data statement) state is not set
-     If getset is 1 state is set to qvalue
-     If getset is 0 state returned in qvalue
-**********************************************************************
-*/
-{
-    static FourByteLong qinit = 0;
-    if(getset == 0) *qvalue = qinit;
-    else qinit = *qvalue;
-}
-
-extern void gssst(const FourByteLong getset, /* not const */ FourByteLong *const qset)
-/*
-**********************************************************************
-     void gssst(FourByteLong getset,FourByteLong *qset)
-          Get or Set whether Seed is Set
-     Initialize to Seed not Set
-     If getset is 1 sets state to Seed Set
-     If getset is 0 returns 1 in qset if Seed Set
-     Else returns 0 in qset
-**********************************************************************
-*/
-{
-    static FourByteLong qstate = 0;
-    if(getset != 0) qstate = 1;
-    else  *qset = qstate;
-}
-
-static FourByteLong ignbin(const FourByteLong n, ConstReal pp) // not used in AutoDock code
-/*
-**********************************************************************
-     FourByteLong ignbin(FourByteLong n,Real pp)
+     FourByteLong ignbin(int n,Real pp)
                     GENerate BINomial random deviate
                               Function
      Generates a single random deviate from a binomial
@@ -796,7 +729,7 @@ static FourByteLong ignbin(const FourByteLong n, ConstReal pp) // not used in Au
 */
 {
 static Real psave = -1.0;
-static FourByteLong nsave = -1;
+static int nsave = -1;
 static FourByteLong ignbin,i,ix,ix1,k,m,mp,T1;
 static Real al,alv,amaxp,c,f,f1,f2,ffm,fm,g,p,p1,p2,p3,p4,q,qn,r,u,v,w,w2,x,x1,
     x2,xl,xll,xlr,xm,xnp,xnpq,xr,ynorm,z,z2;
@@ -947,7 +880,7 @@ S170:
     return ignbin;
 }
 
-FourByteLong ignnbn(const FourByteLong n, ConstReal p) // not used in AutoDock code
+FourByteLong ignnbn(const int n, ConstReal p) // not used in AutoDock code
 /*
 **********************************************************************
  
@@ -970,8 +903,8 @@ FourByteLong ignnbn(const FourByteLong n, ConstReal p) // not used in AutoDock c
 **********************************************************************
 */
 {
-static FourByteLong ignnbn;
-static Real y,a,r;
+static int ignnbn;
+Real y,a,r;
 /*
      ..
      .. Executable Statements ..
@@ -995,7 +928,7 @@ static Real y,a,r;
     return ignnbn;
 }
 
-static FourByteLong ignpoi(ConstReal mu)
+static FourByteLong ignpoi(ConstReal mu) // ignpoi() not used in AutoDock code
 /*
 **********************************************************************
      FourByteLong ignpoi(Real mu)
@@ -1046,21 +979,21 @@ static FourByteLong ignpoi(ConstReal mu)
      SEPARATION OF CASES A AND B
 */
 {
-static Real a0 = -0.5;
-static Real a1 = 0.3333333;
-static Real a2 = -0.2500068;
-static Real a3 = 0.2000118;
-static Real a4 = -0.1661269;
-static Real a5 = 0.1421878;
-static Real a6 = -0.1384794;
-static Real a7 = 0.125006;
+const static Real a0 = -0.5;
+const static Real a1 = 0.3333333;
+const static Real a2 = -0.2500068;
+const static Real a3 = 0.2000118;
+const static Real a4 = -0.1661269;
+const static Real a5 = 0.1421878;
+const static Real a6 = -0.1384794;
+const static Real a7 = 0.125006;
 static Real muold = 0.0;
 static Real muprev = 0.0;
 static Real fact[10] = {
     1.0,1.0,2.0,6.0,24.0,120.0,720.0,5040.0,40320.0,362880.0
 };
-static FourByteLong ignpoi,j,k,kflag,l,m;
-static Real b1,b2,c,c0,c1,c2,c3,d,del,difmuk,e,fk,fx,fy,g,omega,p,p0,px,py,q,s,
+FourByteLong ignpoi,j,k,kflag,l,m;
+Real b1,b2,c,c0,c1,c2,c3,d,del,difmuk,e,fk,fx,fy,g,omega,p,p0,px,py,q,s,
     t,u,v,x,xx,pp[35];
 
     if(mu == muprev) goto S10;
@@ -1234,10 +1167,10 @@ S180:
     return ignpoi;
 }
 
-extern FourByteLong ignuin(const FourByteLong low, const FourByteLong high)
+extern int ignuin(const int low, const int high)
 /*
 **********************************************************************
-     FourByteLong ignuin(FourByteLong low,FourByteLong high)
+     int ignuin(int low,int high)
                GeNerate Uniform INteger
                               Function
      Generates an integer uniformly distributed between LOW and HIGH.
@@ -1253,22 +1186,15 @@ extern FourByteLong ignuin(const FourByteLong low, const FourByteLong high)
 */
 {
 #define maxnum 2147483561L
-static FourByteLong ignuin,ign,maxnow,range,ranp1;
+FourByteLong ign,maxnow,range,ranp1;
 
-    if(!(low > high)) goto S10;
-    stop("ERROR low > high in ignuin");
+    if(low > high) stop("ERROR low > high in ignuin");
 
-S10:
     range = high-low;
-    if(!(range > maxnum)) goto S20;
-    stop("ERROR high - low too large in ignuin");
+    if((range > maxnum)) stop("ERROR high - low too large in ignuin");
 
-S20:
-    if(!(low == high)) goto S30;
-    ignuin = low;
-    return ignuin;
+    if(low == high) return low;
 
-S30:
 /*
      Number to be generated should be in range 0..RANGE
      Set MAXNOW so that the number of integers in 0..MAXNOW is an
@@ -1278,23 +1204,20 @@ S30:
     maxnow = maxnum/ranp1*ranp1;
 S40:
     ign = ignlgi()-1;
-    if(!(ign <= maxnow)) goto S50;
-    ignuin = low+ign%ranp1;
-    return ignuin;
-S50:
+    if((ign <= maxnow)) return low+ign%ranp1;
     goto S40;
 #undef maxnum
 #undef err1
 #undef err2
 }
 
-FourByteLong lennob(const char *const str) // not used in AutoDock code
+int lennob(const char *const str) // not used in AutoDock code
 /* 
 Returns the length of str ignoring trailing blanks but not 
 other white space.
 */
 {
-FourByteLong i, i_nb;
+int i, i_nb;
 
 for (i=0, i_nb= -1L; *(str+i); i++)
     if ( *(str+i) != ' ' ) i_nb = i;
@@ -1316,12 +1239,13 @@ extern FourByteLong mltmod(const FourByteLong a, const FourByteLong s, const Fou
 **********************************************************************
 */
 {
-#define h 32768L
-static FourByteLong mltmod,a0,a1,k,p,q,qh,rh;
+FourByteLong mltmod,a0,a1,k,p,q,qh,rh;
 /*
      H = 2**((b-2)/2) where b = 32 because we are using a 32 bit
       machine. On a different machine recompute H
 */
+//#define h ((sizeof a0 == 4) ? (1<<((32-2)/2))  : (1<<((64-2)/2))  )
+#define h 32768L
     if((a <= 0 || a >= m || s <= 0 || s >= m)) {
      char errmsg[300];
     fputs(" a, m, s out of order in mltmod - ABORT!",stderr);
@@ -1396,8 +1320,7 @@ S130:
     goto S130;
 S150:
 S140:
-    mltmod = p;
-    return mltmod;
+    return p;
 #undef h
 }
 
@@ -1427,7 +1350,7 @@ void phrtsd(const char *const phrase, FourByteLong *const seed1, FourByteLong *c
 */
 {
 
-static char table[] =
+const static char table[] =
 "abcdefghijklmnopqrstuvwxyz\
 ABCDEFGHIJKLMNOPQRSTUVWXYZ\
 0123456789\
@@ -1435,11 +1358,11 @@ ABCDEFGHIJKLMNOPQRSTUVWXYZ\
 
 FourByteLong ix;
 
-static FourByteLong twop30 = 1073741824L;
-static FourByteLong shift[5] = {
+const static FourByteLong twop30 = 1073741824L;
+const static FourByteLong shift[5] = {
     1L,64L,4096L,262144L,16777216L
 };
-static FourByteLong i,ichr,j,lphr,values[5];
+FourByteLong i,ichr,j,lphr,values[5];
 FourByteLong lennob(const char *const str);
 
     *seed1 = 1234567890L;
@@ -1464,12 +1387,14 @@ FourByteLong lennob(const char *const str);
 #undef twop30
 }
 
+#ifdef RANF_AS_FUNCTION
+ /* changed to macro in ranlib.h to avoid extra function call */
 extern Real ranf(void)
 /*
 **********************************************************************
      Real ranf(void)
                 RANDom number generator as a Function
-     Returns a random Realing point number from a uniform distribution
+     Returns a random floating point number from a uniform distribution
      over 0 - 1 (endpoints of this interval are not returned) using the
      current generator
      This is a transcription from Pascal to Fortran of routine
@@ -1480,7 +1405,7 @@ extern Real ranf(void)
 **********************************************************************
 */
 {
-static Real ranf;
+Real ranf;
 /*
      4.656613057E-10 is 1/M1  M1 is set in a data statement in IGNLGI
       and is currently 2147483563. If M1 changes, change this also.
@@ -1488,11 +1413,12 @@ static Real ranf;
     ranf = ignlgi()*4.656613057E-10;
     return ranf;
 }
+#endif
 
-void setgmn(const Real *const meanv, Real *const covm, const FourByteLong p, Real *const parm) // not used in AutoDock code
+void setgmn(const Real *const meanv, Real *const covm, const int p, Real *const parm) // not used in AutoDock code
 /*
 **********************************************************************
-     void setgmn(Real *meanv,Real *covm,FourByteLong p,Real *parm)
+     void setgmn(Real *meanv,Real *covm,int p,Real *parm)
             SET Generate Multivariate Normal random deviate
                               Function
       Places P, MEANV, and the Cholesky factoriztion of COVM
@@ -1513,17 +1439,17 @@ void setgmn(const Real *const meanv, Real *const covm, const FourByteLong p, Rea
 **********************************************************************
 */
 {
-extern void spofa(/* not const */ Real *const a,const FourByteLong lda,const FourByteLong n,/* not const */ FourByteLong *const info);
+extern void spofa(/* not const */ Real *const a,const int lda,const int n,/* not const */ int *const info);
 /* not used: static FourByteLong T1; */
-static FourByteLong i,icount,info,j,D2,D3,D4,D5;
+int i,icount,info,j,D2,D3,D4,D5;
     /* not used: T1 = p*(p+3)/2+1; */
 /*
      TEST THE INPUT
 */
-    if(!(p <= 0)) goto S10;
-    fprintf(stderr,"Value of P: %12ld\n",p);
-    stop("ERROR P nonpos in setgmn");
-S10:
+    if(p <= 0) {
+      fprintf(stderr,"Value of P: %d\n",p);
+      stop("ERROR P nonpos in setgmn");
+      }
     *parm = p;
 /*
      PUT P AND MEANV INTO PARM
@@ -1585,9 +1511,9 @@ static Real sexpo(void)
 static Real q[8] = {
     0.6931472,0.9333737,0.9888778,0.9984959,0.9998293,0.9999833,0.9999986,1.0
 };
-static FourByteLong i;
-static Real sexpo,a,u,ustar,umin;
-static Real *q1 = q;
+FourByteLong i;
+Real a,u,ustar,umin;
+Real *q1 = q;
     a = 0.0;
     u = ranf();
     goto S30;
@@ -1598,8 +1524,7 @@ S30:
     if(u <= 1.0) goto S20;
     u -= 1.0;
     if(u > *q1) goto S60;
-    sexpo = a+u;
-    return sexpo;
+    return a+u;
 S60:
     i = 1;
     ustar = ranf();
@@ -1609,8 +1534,7 @@ S70:
     if(ustar < umin) umin = ustar;
     i += 1;
     if(u > *(q+i-1)) goto S70;
-    sexpo = a+umin**q1;
-    return sexpo;
+    return a+umin**q1;
 }
 
 static Real sgamma(ConstReal a)
@@ -1666,29 +1590,29 @@ static Real sgamma(ConstReal a)
      SQRT32 IS THE SQUAREROOT OF 32 = 5.656854249492380
 */
 {
-static Real q1 = 4.166669E-2;
-static Real q2 = 2.083148E-2;
-static Real q3 = 8.01191E-3;
-static Real q4 = 1.44121E-3;
-static Real q5 = -7.388E-5;
-static Real q6 = 2.4511E-4;
-static Real q7 = 2.424E-4;
-static Real a1 = 0.3333333;
-static Real a2 = -0.250003;
-static Real a3 = 0.2000062;
-static Real a4 = -0.1662921;
-static Real a5 = 0.1423657;
-static Real a6 = -0.1367177;
-static Real a7 = 0.1233795;
-static Real e1 = 1.0;
-static Real e2 = 0.4999897;
-static Real e3 = 0.166829;
-static Real e4 = 4.07753E-2;
-static Real e5 = 1.0293E-2;
+const static Real q1 = 4.166669E-2;
+const static Real q2 = 2.083148E-2;
+const static Real q3 = 8.01191E-3;
+const static Real q4 = 1.44121E-3;
+const static Real q5 = -7.388E-5;
+const static Real q6 = 2.4511E-4;
+const static Real q7 = 2.424E-4;
+const static Real a1 = 0.3333333;
+const static Real a2 = -0.250003;
+const static Real a3 = 0.2000062;
+const static Real a4 = -0.1662921;
+const static Real a5 = 0.1423657;
+const static Real a6 = -0.1367177;
+const static Real a7 = 0.1233795;
+const static Real e1 = 1.0;
+const static Real e2 = 0.4999897;
+const static Real e3 = 0.166829;
+const static Real e4 = 4.07753E-2;
+const static Real e5 = 1.0293E-2;
+const static Real sqrt32 = 5.656854;
+static Real sgamma,s2,s,d,t,x,u,r,q0,b,si,c,v,q,e,w,p;
 static Real aa = 0.0;
 static Real aaa = 0.0;
-static Real sqrt32 = 5.656854;
-static Real sgamma,s2,s,d,t,x,u,r,q0,b,si,c,v,q,e,w,p;
     if(a == aa) goto S10;
     if(a < 1.0) goto S120;
 /*
@@ -1856,42 +1780,42 @@ static Real snorm(void)
      H(K) ARE ACCORDING TO THE ABOVEMENTIONED ARTICLE
 */
 {
-static Real a[32] = {
+static const Real a[32] = {
     0.0,3.917609E-2,7.841241E-2,0.11777,0.1573107,0.1970991,0.2372021,0.2776904,
     0.3186394,0.36013,0.4022501,0.4450965,0.4887764,0.5334097,0.5791322,
     0.626099,0.6744898,0.7245144,0.7764218,0.8305109,0.8871466,0.9467818,
     1.00999,1.077516,1.150349,1.229859,1.318011,1.417797,1.534121,1.67594,
     1.862732,2.153875
 };
-static Real d[31] = {
+static const Real d[31] = {
     0.0,0.0,0.0,0.0,0.0,0.2636843,0.2425085,0.2255674,0.2116342,0.1999243,
     0.1899108,0.1812252,0.1736014,0.1668419,0.1607967,0.1553497,0.1504094,
     0.1459026,0.14177,0.1379632,0.1344418,0.1311722,0.128126,0.1252791,
     0.1226109,0.1201036,0.1177417,0.1155119,0.1134023,0.1114027,0.1095039
 };
-static Real t[31] = {
+static const Real t[31] = {
     7.673828E-4,2.30687E-3,3.860618E-3,5.438454E-3,7.0507E-3,8.708396E-3,
     1.042357E-2,1.220953E-2,1.408125E-2,1.605579E-2,1.81529E-2,2.039573E-2,
     2.281177E-2,2.543407E-2,2.830296E-2,3.146822E-2,3.499233E-2,3.895483E-2,
     4.345878E-2,4.864035E-2,5.468334E-2,6.184222E-2,7.047983E-2,8.113195E-2,
     9.462444E-2,0.1123001,0.136498,0.1716886,0.2276241,0.330498,0.5847031
 };
-static Real h[31] = {
+static const Real h[31] = {
     3.920617E-2,3.932705E-2,3.951E-2,3.975703E-2,4.007093E-2,4.045533E-2,
     4.091481E-2,4.145507E-2,4.208311E-2,4.280748E-2,4.363863E-2,4.458932E-2,
     4.567523E-2,4.691571E-2,4.833487E-2,4.996298E-2,5.183859E-2,5.401138E-2,
     5.654656E-2,5.95313E-2,6.308489E-2,6.737503E-2,7.264544E-2,7.926471E-2,
     8.781922E-2,9.930398E-2,0.11556,0.1404344,0.1836142,0.2790016,0.7010474
 };
-static FourByteLong i;
-static Real snorm,u,s,ustar,aa,w,y,tt;
+FourByteLong i;
+Real snorm,u,s,ustar,aa,w,y,tt;
     u = ranf();
     s = 0.0;
     if(u > 0.5) s = 1.0;
     u += (u-s);
     u = 32.0*u;
     i = (FourByteLong) (u);
-    if(i == 32) i = 31;
+    if(i >= 32) i = 31;  // MP 2013 was i == 32
     if(i == 0) goto S100;
 /*
                                 START CENTER
