@@ -1,6 +1,6 @@
 /*
 
- $Id: simanneal.cc,v 1.43 2012/08/18 00:00:29 mp Exp $
+ $Id: simanneal.cc,v 1.50 2014/07/10 23:27:37 mp Exp $
 
  AutoDock  
 
@@ -43,7 +43,7 @@ Copyright (C) 2009 The Scripps Research Institute. All rights reserved.
 extern char *programname;
 
 
-void simanneal ( int   *const Addr_nconf,
+void simanneal ( int   *const Addr_nconf, /* absolute serial number of "run", modified here */
                 const int   Nnb,
 		int Nnb_array[3],
 		GroupEnergy *group_energy,
@@ -64,8 +64,8 @@ void simanneal ( int   *const Addr_nconf,
                 const Boole B_either,
 		EnergyComponent	*peratomE,
                 const int   NcycMax,
-                const int   irunmax,
-		FourByteLong seed[2],
+                const int   nruns,
+		FourByteLong runseed[][2], /* [MAX_RUNS] */
                 const Clock& jobStart,
                 #include "map_declare.h"
                 const int   naccmax,
@@ -170,7 +170,6 @@ void simanneal ( int   *const Addr_nconf,
     Real etot = 0.0;
     Real inv_RT = 0.;
     Real qtwStep;
-    Real rsqC1C2;
     Real RT = 616.;
     Real torTmp;
     Real torStep;
@@ -183,10 +182,7 @@ void simanneal ( int   *const Addr_nconf,
 
     int count = 0;
     int Itor = 0;
-    int icycle = -1;
-    int icycle1 = -1;
-    int irun = -1;
-    int irun1 = -1;
+    int irun;
     int indx;
 
     register int i = 0;
@@ -195,8 +191,6 @@ void simanneal ( int   *const Addr_nconf,
     register int nAccProb = 0;
     register int nedge = 0;
     register int nrej = 0;
-    register int ntot = 1;
-    register int xyz = 0;
 
     struct tms tms_cycStart;
     struct tms tms_cycEnd;
@@ -226,7 +220,7 @@ void simanneal ( int   *const Addr_nconf,
             prStr( error_message, "\n%s: can't create trajectory file %s\n", programname, FN_trj);
             pr_2x( stderr, logFile, error_message );
             jobEnd = times( &tms_jobEnd );
-            timesys( jobEnd - jobStart, &tms_jobStart, &tms_jobEnd );
+            timesys( jobEnd - jobStart, &tms_jobStart, &tms_jobEnd, logFile);
             pr_2x( logFile, stderr, UnderLine );
             stop(error_message);
         }/*END PROGRAM*/
@@ -234,24 +228,28 @@ void simanneal ( int   *const Addr_nconf,
 
 /* Begin the automated docking simulation, ===================================*/
 
-    if(outlev>=LOGBASIC) {
+    if(outlev>LOGBASIC) {
     pr( logFile, "\n\n\t\tBEGINNING MONTE CARLO SIMULATED ANNEALING\n");
     pr( logFile, "     \t\t_________________________________________\n\n\n\n" );
     }
 
 
-    for ( irun = 0;  irun < irunmax;  irun++ ) { /*===========================*/
+    int nconf = *(Addr_nconf); /* absolute serial number of this run */
+    for ( irun = 0;  irun < nruns;  irun++ ) { /*===========================*/
 
-        irun1 = 1 + irun;
+	/* set RNG seed using global run number, see main.cc and com.cc */
+	if(nconf==0&&irun==0) getsd(&runseed[nconf][0], &runseed[nconf][1]);
+	else setsd(runseed[nconf+irun][0], runseed[nconf+irun][1]); 
 
 
-        if (outlev >= LOGFORADT) {
+        if (outlev >= LOGRUNV) 
             pr(logFile, "\n\tINITIALIZING AUTOMATED DOCKING SIMULATION\n" );
-            pr( logFile, "RUN %d\n", irun1); // MP removed ... OK? TODO
-        }
+        pr( logFile, "Run: %d Seed: %ld %ld [ Run %d of %d SA ]\n", nconf+irun+1,
+		(long)runseed[nconf+irun][0], (long)runseed[nconf+irun][1],
+		irun+1, nruns);
 
         if ( B_writeTrj ) {
-            pr( FP_trj, "ntorsions %d\nrun %d\n", ntor, irun1 );
+            pr( FP_trj, "ntorsions %d\nrun %d\n", ntor, nconf+irun+1 );
             fflush( FP_trj );
         }
 
@@ -262,7 +260,7 @@ void simanneal ( int   *const Addr_nconf,
                      peratomE, ptr_ad_energy_tables, B_calcIntElec,
                      map, natom, Nnb, Nnb_array, group_energy, 
 		     true_ligand_atoms, nonbondlist,
-                     ntor, tlist, type, vt, irun1, MaxRetries,
+                     ntor, tlist, type, vt, nconf+irun, MaxRetries,
                      torsFreeEnergy, ligand_is_inhibitor,
                      ignore_inter,
                      B_include_1_4_interactions, scale_1_4, scale_eintermol,
@@ -288,7 +286,7 @@ void simanneal ( int   *const Addr_nconf,
             pr( logFile, "\n\n\t\tBEGINNING SIMULATED ANNEALING");
             pr( logFile, "\n\t\t_____________________________\n\n");
             pr( logFile, "\n      \t      \tMinimum     Average     | Acc/    Accepted:    Rejected:     |          |  xyz-Translation  |        Time:        \n");
-              pr( logFile, "Run:  \tCycle:\tEnergy:     Energy:     |   /Rej: Down:  Up:   Total: Edge:  |   RT:    |   of Min.Energy   |  Real, CPU, System  \n" );
+              pr( logFile, "Run#  \tCycle:\tEnergy:     Energy:     |   /Rej: Down:  Up:   Total: Edge:  |   RT:    |   of Min.Energy   |  Real, CPU, System  \n" );
               pr( logFile, "______\t______\t___________ ___________ | ______ ______ ______ ______ ______ | ________ | _________________ | ____________________\n" );
 /*                                          12345678901 12345678901   123456 123456 123456 123456 123456   12345678   12345 12345 12345
 **                         "%D /%D\T%D /%D\T%+11.2F     %+11.2F       %6.2F  %6D    %6D    %6D    %6D      %8.1F     %5.2F %5.2F %5.2F   ",
@@ -298,14 +296,15 @@ void simanneal ( int   *const Addr_nconf,
         fflush(logFile);
 /*____________________________________________________________________________*/
 
-        for ( icycle = 0;  icycle < NcycMax;  icycle++ ) {
+        for ( int icycle = 0;  icycle < NcycMax;  icycle++ ) {
 
+	    int ntot;
             cycStart = times( &tms_cycStart );
 
-            icycle1 = icycle + (ntot = 1);
+	    ntot = 1;
             B_inRange = (icycle >= trj_cyc_min) && (icycle <= trj_cyc_max);
             if ( B_writeTrj && B_inRange ) {
-                pr( FP_trj, "cycle %d\ntemp %f\n", icycle1, RT );
+                pr( FP_trj, "cycle %d\ntemp %f\n", icycle+1, RT );
                 fflush(  FP_trj  );
             }
             nAcc = nAccProb = nacc = nrej = nedge = 0;
@@ -330,7 +329,8 @@ void simanneal ( int   *const Addr_nconf,
 		    outlev, logFile);
 
                 if (B_constrain) {
-                    for (xyz = 0;  xyz < SPACE;  xyz++) {
+		    Real rsqC1C2;
+                    for (int xyz = 0;  xyz < SPACE;  xyz++) {
                         d[xyz] = crd[atomC1][xyz] - crd[atomC2][xyz];
                     }
                     rsqC1C2 = sqhypotenuse(d[X],  d[Y], d[Z]);
@@ -401,7 +401,7 @@ void simanneal ( int   *const Addr_nconf,
                                );
 
                         if (B_isGaussTorCon) {
-                            /*** This looks wrong... for (Itor = 0; Itor <= ntor; Itor++) { MP ***/
+                            /*** This looks wrong... for (Itor = 0; Itor <= ntor; Itor++)   MP ***/
                             for (Itor = 0; Itor < ntor; Itor++) {
                                 if (B_isTorConstrained[Itor] == 1) {
                                     indx = RadiansToDivs( sNow.tor[Itor] );
@@ -504,7 +504,7 @@ void simanneal ( int   *const Addr_nconf,
                          peratomE, ptr_ad_energy_tables, B_calcIntElec,
                          map, natom, 
 			 Nnb, Nnb_array, group_energy, true_ligand_atoms,
-			 nonbondlist, ntor, tlist, type, vt, irun1, MaxRetries,
+			 nonbondlist, ntor, tlist, type, vt, nconf+irun, MaxRetries,
                          torsFreeEnergy, ligand_is_inhibitor,
                          ignore_inter,
                          B_include_1_4_interactions, scale_1_4, scale_eintermol,
@@ -524,9 +524,9 @@ void simanneal ( int   *const Addr_nconf,
                 */
                 if (outlev >= LOGRUNV) {
                     /*pr(logFile, "\n"); / *###*/
-                    pr( logFile, "%d /%d\t%d /%d\t%+11.2f %+11.2f   %6.2f %6d %6d %6d %6d   %8.1f   %5.2f %5.2f %5.2f   ", irun1, irunmax, icycle1, NcycMax, eMin, etot/ntot, (nrej!=0) ? (Real)nacc/nrej : 999.99, nAcc, nAccProb, nrej, nedge, RT, sMin.T.x, sMin.T.y, sMin.T.z );
+                    pr( logFile, "%d \t%d /%d\t%+11.2f %+11.2f   %6.2f %6d %6d %6d %6d   %8.1f   %5.2f %5.2f %5.2f   ", nconf+irun+1, icycle+1, NcycMax, eMin, etot/ntot, (nrej!=0) ? (Real)nacc/nrej : 999.99, nAcc, nAccProb, nrej, nedge, RT, sMin.T.x, sMin.T.y, sMin.T.z );
                     cycEnd = times( &tms_cycEnd );
-                    timesys( cycEnd - cycStart, &tms_cycStart, &tms_cycEnd );
+                    timesys( cycEnd - cycStart, &tms_cycStart, &tms_cycEnd, logFile);
                     if (outlev > LOGRUNV ) {
                         pr( logFile, "\tEnergy:   \tState:\n\t__________\t____________________________________________________________\nMinimum\t%+6.2f\t(%+.2f,%+.2f,%+.2f), q = [x,y,z,w] = [%5.1f deg, (%+.2f,%+.2f,%+.2f)],\n", eMin, sMin.T.x, sMin.T.y, sMin.T.z, sMin.Q.x, sMin.Q.y, sMin.Q.z, sMin.Q.w );
                         pr( logFile, "\nLast\t%+6.2f\t(%+.2f,%+.2f,%+.2f), q = [x,y,z,w] = [%5.1f deg, (%+.2f,%+.2f,%+.2f)],\n", eLast, sLast.T.x, sLast.T.y, sLast.T.z, sLast.Q.x, sLast.Q.y, sLast.Q.z, sLast.Q.w );
@@ -586,8 +586,10 @@ void simanneal ( int   *const Addr_nconf,
             sSave.tor[i] = WrpRad( ModRad( sSave.tor[i] ) );
         }
    
-        pr( logFile, "\tFINAL DOCKED STATE\n" );
-        pr( logFile, "Run Number %d\nFinal Energy = %+.2f\n", irun1, eLast);
+	if(outlev>=LOGRUNV) pr( logFile, "\tFINAL DOCKED STATE\n" );
+        if(outlev>=LOGRUNVV)pr( logFile, "Run Number %d\n", nconf+irun+1);
+        if(outlev>=LOGRUNVV)pr( logFile, "Final Energy = %+.2f\n", eLast);
+	pr(logFile,"Final-Value: %.3f\n", eLast);
 
 	if(outlev>=LOGRUNV) {
         pr( logFile, "Final Translation = %.2f, %.2f, %.2f\n", sSave.T.x, sSave.T.y, sSave.T.z );
@@ -597,9 +599,9 @@ void simanneal ( int   *const Addr_nconf,
         pr( logFile, "Final Rotation Angle = %5.1f deg\n", RadiansToDegrees(WrpRad( ModRad(aa.ang))) );
 	}
 
-        copyState( &sHist[ *Addr_nconf ], sSave );
+        copyState( &sHist[ nconf+irun ], sSave );
 
-        for (i=0; i<ntor; i++) sHist[ *Addr_nconf ].tor[i] = sSave.tor[i];
+        for (i=0; i<ntor; i++) sHist[ nconf+irun ].tor[i] = sSave.tor[i];
 
         if (ntor > 0 && outlev>=LOGRUNVV) {
 	    pr( logFile, "Final Torsions:\n" );
@@ -607,7 +609,7 @@ void simanneal ( int   *const Addr_nconf,
                 torTmp = RadiansToDegrees( sSave.tor[i] );
                 torTmp = ModDeg( torTmp );
                 torTmp = WrpDeg( torTmp );
-                sHist[ *Addr_nconf ].tor[i] = sSave.tor[i];
+                sHist[ nconf+irun ].tor[i] = sSave.tor[i];
                 pr( logFile, "          %2d = %7.2f deg", i+1, torTmp);
                 if (B_isTorConstrained[i] && B_ShowTorE) {
                     pr(logFile, ", Energetic penalty = %uhd\n", US_TorE[i]);
@@ -635,7 +637,7 @@ void simanneal ( int   *const Addr_nconf,
                     NULL_ENERGY_BREAKDOWN);
 
 	if(outlev>=LOGMIN)
-        writePDBQT( irun, seed, FN_ligand, FN_dpf, lig_center, sSave, ntor,
+        writePDBQT( nconf+irun, runseed[nconf], FN_ligand, FN_dpf, lig_center, sSave, ntor,
                 &eintra, &einter, natom, atomstuff, crd, peratomE,
                 charge, abs_charge, qsp_abs_charge,
                 ligand_is_inhibitor, torsFreeEnergy, 
@@ -657,11 +659,11 @@ void simanneal ( int   *const Addr_nconf,
         } else {
             local_unbound_internal_FE = unbound_internal_FE;
         }
-        // originally: econf[(*Addr_nconf)] = eLast;
-        econf[(*Addr_nconf)] = eintra + einter + torsFreeEnergy - local_unbound_internal_FE;
-        ++(*Addr_nconf);
+        // originally: econf[nconf+irun] = eLast;
+        econf[nconf+irun] = eintra + einter + torsFreeEnergy - local_unbound_internal_FE;
 
     } /* Loop over runs ======================================================*/
+    *Addr_nconf += nruns;
 
     if ( B_writeTrj ) {
             fclose( FP_trj );
